﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace XilinxBitFile
{
    /// <summary>
    /// Class that takes out contents and length of 
    /// Xilinx .bit file when file name is given. 
    /// </summary>
    public class BitFile
    {
        /// <summary>
        /// Bitstream Length (bytes)
        /// </summary>
        public int Length ;

        /// <summary>
        /// Contents of Bitstream
        /// </summary>
        public byte[] Content ;

        /// <summary>
        /// Reading .bit file and parse
        /// </summary>
        /// <param name="FileName"></param>
        public BitFile(string FileName)
        {
            int hf; // Header Field Count
            int bytes; // Field Bytes
            int len; // Bitstream Part Length

            try
            {
                using (FileStream fs = new FileStream(FileName, FileMode.Open))
                {
                    int recv ;

                    hf = 1;
                    bytes = 0;
                    len = 0;
                    while ((recv = fs.ReadByte()) >= 0)
                    {
                        // Header
                        if (hf <= 3)
                        {
                            // No Key Before Field 3
                            if (bytes == 0x00)
                            {
                                len = recv;
                                bytes++;
                            }
                            else if (bytes == 0x01)
                            {
                                len = (len * 0x100) + recv;
                                bytes++;
                            }
                            else
                            {
                                // Field 1-3
                                len--;
                                if (len == 0)
                                {
                                    bytes = 0;
                                    hf++; // Next Field
                                }
                            }
                        }
                        else
                        {
                            // After Field 4
                            if (bytes == 0x00)
                            {
                                bytes++;
                            }
                            else if (bytes == 0x01)
                            {
                                len = recv;
                                bytes++;
                            }
                            else if (bytes == 0x02)
                            {
                                len = (len << 8) + recv;
                                bytes++;
                            }
                            else
                            {
                                // 4 bytes only Field 7
                                if (hf == 7)
                                {
                                    len = (len << 8) + recv;
                                    bytes++;
                                    if (bytes == 0x05)
                                    {
                                        break;
                                    }
                                    continue;
                                }
                                // Field 4-6
                                len--;
                                if (len == 0)
                                {
                                    bytes = 0;
                                    hf++; // Next Field
                                }
                            }
                        }
                    }

                    // Make Byte Array
                    Length = len ;
                    Content = new byte[len] ;
	                for (int count=0;count <len;count++) {		
    	                Content[count] =(byte)fs.ReadByte();
                    }                                 
                }
            }
            catch
            {
                // Read or Open Failed
                Length = 0;
                Content = new byte[0];
            }
        }
    }
}
