﻿using System;
using System.Threading;
using System.Windows.Forms;

using ZZBase;
using ZZComm;

using XilinxBitFile;

namespace avnettest
{
    /// <summary>
    /// AVNET SP3A EVAL Configure Test Program
    ///
    /// Copyright(C) Moxi Users Group 2008 , All Rights Reserved Under GPL.
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// SerialPort Object
        /// </summary>
        private ZZCommSerial SerialPort1;

        /// <summary>
        /// RXBuffer Object
        /// </summary>
        private ZZCommData RxBuffer1;

        /// <summary>
        /// If set true, Async Rxdata will be delegated to UI Thread
        /// </summary>
        private bool AutoPrint = true;

        public Form1()
        {
            InitializeComponent();

            // Read Port from User.config
            Properties.Settings psd = Properties.Settings.Default;

            // Making Serial Port and RXBuffer
            SerialPort1 = new ZZCommSerial("PORT1",psd.Port1, int.Parse(psd.BitRate1), "NONE");
            RxBuffer1 = new ZZCommData("RXBUFFER1");

            // Pseudo-Async Rx Handler
            SerialPort1.Event += RxBuffer1.RxEvent ;
            SerialPort1.Event += this.RxEvent1 ;

            // Open Serial Port
            try
            {
                SerialPort1.Open();
            }
            catch
            {
                textBox1.AppendText("Serial Port Open Failed. Please Change COMxx in Property-Box and ReRun.\n");
            }

            // Comm Parameter - Property Grid
            propertyGrid1.SelectedObject = SerialPort1;
        }

        /// <summary>
        /// Slave Serial Configration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            BitFile bf = new BitFile(openFileDialog1.FileName);
            if (bf.Length == 0)
            {
                MessageBox.Show("Bit File Open Falied.", "Warning");
                return;
            }

            // Async AutoPrint OFF
            AutoPrint = false;

            //
            // The following instructions are from....
            //
            // *Spartan3A Evaluation Board PSoC Software User Guide
            // *Spartan3A Evaluation Board Programming Algorithms
            //
            // by AVNET
            //
            
            // 1.Load SPI configuration
            sp3a_eval_send("load_config 1");

            // 2.Drive PROG low
            sp3a_eval_send("drive_prog 0");

            // 3.Drive M[2:0] to 1:1:1
            sp3a_eval_send("drive_mode 7");

            // 4.Drive PSOC_SPI_MODE high
            sp3a_eval_send("spi_mode 1");

            // 5.Drive PROG high
            sp3a_eval_send("drive_prog 1");

            // 6.Wait for INIT to go high
            sp3a_eval_send("read_init"); // *** NO CHECK ***

            // 7.Drive M[2:0] to tri-state
            sp3a_eval_send("drive_mode 8");

            // 8.Assert the FPGA reset to hold the FPGA application in reset until the PSoC has had time to switch configurations
            sp3a_eval_send("fpga_rst 1");
            
            // 9.ss_program <# bytes in the .bit file>            
            sp3a_eval_send(String.Format("ss_program {0}",bf.Length)) ;

            // 10.Send bytes from .bit file
            SerialPort1.Send(bf.Content);

            // Clear too much ack
            RxBuffer1.Clear();

            // 11.Check INIT is still high
            sp3a_eval_send("read_init"); // *** NO CHECK ***
            
            // 12.Check DONE is high
            sp3a_eval_send("read_done"); // *** NO CHECK ***

            // 13.Drive PSOC_SPI_MODE low
            sp3a_eval_send("spi_mode 0");

            // 14.Load UART configuration
            sp3a_eval_send("load_config 0");

            // 15.Deassert FPGA reset
            sp3a_eval_send("fpga_rst 0");

            AutoPrint = true;
        }

        /// <summary>
        /// Send Command To PSoC
        /// </summary>
        /// <param name="b"></param>
        private void sp3a_eval_send(String s)
        {
            // Print Command to Textbox
            textBox1.AppendText("\r\n" + s + "-->");
            SerialPort1.Send(ZZUtil.StrToByte(s));
            SerialPort1.Send(new byte[1] { 0 });

            Thread.Sleep(50); // 50 is Groundless figure
            textBox1.AppendText(RxBuffer1.RxData);
            RxBuffer1.Clear();
        }

        /// <summary>
        /// Pseudo-Async RX Handler (called from *NOT* UI Thread)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RxEvent1(object sender, EventArgs e)
        {
            if (AutoPrint)
            {
                // textBox1.AppendText(s); // UI Thread
                textBox1.BeginInvoke(
                    (ZZUtil.MethodInvoker1)(m => textBox1.AppendText(m.ToString())),
                    new object[] { RxBuffer1.RxData }); // Not UI Thread
                RxBuffer1.Clear();   
            }
        }

        /// <summary>
        /// Save param at closing window.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // get param
            Properties.Settings psd = Properties.Settings.Default;
                
            // save param
            psd.Port1       = SerialPort1.Port;
            psd.BitRate1 = SerialPort1.Bitrate;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// FPGA Reset (= Touch EF4 Button)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click_1(object sender, EventArgs e)
        {
            AutoPrint = false;
            sp3a_eval_send("load_config 1");
            sp3a_eval_send("fpga_rst 1");
            sp3a_eval_send("load_config 0");
            //
            sp3a_eval_send("load_config 1");
            sp3a_eval_send("fpga_rst 0");
            sp3a_eval_send("load_config 0");
            AutoPrint = true;
        }

        /// <summary>
        /// Send Character
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            int key = e.KeyChar;
            SerialPort1.Send(new byte[1] { (byte)key });

            // drop key (when you need localecho , change this "false")
            e.Handled = true;
        }
    }
}
